@@ -7,11 +7,6 @@ use regex::CaptureMatches;
 /// The lifetime parameter `'t` refers to the lifetime of the scanned text.
 ///
 pub struct TokenIter<'t> {
-    /// Start position in the input text as byte offset.
-    /// Can be greater than zero, if `self` was created during a
-    /// scanner state switch before.
-    start_pos: usize,
-
     /// Line number
     line: usize,
 
@@ -43,7 +38,6 @@ impl<'t> TokenIter<'t> {
             .map(String::from)
             .collect();
         TokenIter {
-            start_pos: 0,
             line: 1,
             col: 1,
             capture_iter: rx.rx.captures_iter(input),
@@ -53,40 +47,17 @@ impl<'t> TokenIter<'t> {
     }
 
     ///
-    /// This function is used to setup a new TokenIter at the current stream
-    /// position (aka scanner state switching) by updating all inner position
-    /// values on the newly created TokenIter object.
-    ///
-    pub(crate) fn switch_to(
-        &self,
-        rx: &'static Tokenizer,
-        input: &'t str,
-        pos: usize,
-    ) -> TokenIter<'t> {
-        let start_pos = self.start_pos + pos;
-        let (_, input) = input.split_at(start_pos);
-        let group_names: Vec<String> = rx
-            .rx
-            .capture_names()
-            .flatten()
-            .filter(|n| n.starts_with('G'))
-            .map(String::from)
-            .collect();
-        TokenIter {
-            start_pos,
-            line: self.line,
-            col: self.col,
-            capture_iter: rx.rx.captures_iter(input),
-            group_names,
-            k: self.k,
-        }
+    pub fn with_position(mut self, line: usize, column: usize) -> Self {
+        self.line = line;
+        self.col = column;
+        self
     }
 
     ///
     /// Counts the occurrences of newlines in the given text.
     /// It is used to update the internal line number.
     ///
-    fn count_nl(&self, s: &str) -> usize {
+    pub(crate) fn count_nl(s: &str) -> usize {
         RX_NEW_LINE.find_iter(s).count()
     }
 
@@ -94,7 +65,7 @@ impl<'t> TokenIter<'t> {
     /// Calculates the column position after the last matched newline.
     /// Is used to update the internal column number.
     ///
-    fn calculate_col(&self, s: &str) -> usize {
+    pub(crate) fn calculate_col(s: &str) -> usize {
         let mut matches = RX_NEW_LINE.find_iter(s).collect::<Vec<_>>();
         let right_most_match = matches.pop().unwrap();
         s.len() - right_most_match.end() + 1
@@ -120,16 +91,16 @@ impl<'t> Iterator for TokenIter<'t> {
                 let column = self.col;
 
                 // Set the inner position behind the scanned token
-                let new_lines = self.count_nl(symbol);
+                let new_lines = Self::count_nl(symbol);
                 let pos = ma.end();
                 self.line += new_lines;
                 self.col = if new_lines > 0 {
-                    self.calculate_col(symbol)
+                    Self::calculate_col(symbol)
                 } else {
                     self.col + length
                 };
                 let token = Token::with(symbol, token_type, line, column, length, pos);
-                trace!("{}", token);
+                trace!("{}, newline count: {}", token, new_lines);
                 Some(token)
             } else {
                 // Error
